/******************************************************************************
 *{@C
 *      Copyright:      2005-2025 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D
 *      Filename:       tcl3dArgv.i
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    SWIG file for wrapping "char **" arguments.
 *
 *****************************************************************************/

%module argv

// Tell SWIG to treat char ** as a special case.

%typemap(in) char ** {
     Tcl_Obj **listobjv;
     Tcl_Size  nitems;
     Tcl_Size  i;
     if (Tcl_ListObjGetElements(interp, $input, &nitems, &listobjv) == TCL_ERROR) {
        return TCL_ERROR;
     }
     $1 = (char **) malloc((nitems+1)*sizeof(char *));
     for (i = 0; i < nitems; i++) {
        $1[i] = Tcl_GetStringFromObj(listobjv[i], (Tcl_Size *)NULL);
     }
     $1[i] = 0;
}

// Cleanup code that will get called after the function call.

%typemap(freearg) char ** {
     if ($1) {
        free($1);
     }
}
